package exampler.fitlibrary;

import junit.framework.TestCase;
import fitlibrary.parse.Table;
import fitlibrary.parse.Row;

public class FilterFixtureTestCase extends TestCase {
    protected static final String TAG = "(<.*>)?";
    static int RESULT_TABLE_NUM_HEADING_ROWS = 2;


    // Utilities

    protected static TableParse makeParsedTable(String tableDescription, String derivedTableFixture,
                                         String[][] dataPart) {
        TableParse constructed = TableParse.fromArray(dataPart);
        constructed.addAsFirstRow(RowParse.fromStrings(new String[] { "description", "examples", "counterexamples" }));
        constructed.addAsFirstRow(RowParse.fromCellStrings(tableDescription, derivedTableFixture));
        return constructed;
    }

    protected static Table asFromFitLibrary(TableParse input) {
        return new Table(input);
    }

    protected static Table makeTable(String derivedTableFixture, String[][] dataPart) {
        TableParse inner = makeParsedTable("irrelevant", derivedTableFixture,
                                           dataPart);
        return asFromFitLibrary(inner);
    }

    protected static Table makeTable(String[][] dataPart) {
        return makeTable("irrelevant fixture name", dataPart);
    }

    protected static BooleanFilterFixture makeAndFixture(String[][] dataRows) {
        return makeAndFixture("irrelevant fixture name", dataRows);
    }

    protected static BooleanFilterFixture makeAndFixture(String derivedTableFixture, String[][] dataRows) {
        BooleanFilterFixture fixture = new AndFilterFixture();
        fixture.useTable(makeTable(derivedTableFixture, dataRows));
        return fixture;
    }

    protected static String[][] noRows() {
        return new String[][] { };
    }

    // Assertions

    protected void assertExpectedTestRows(String[][] expected, TableParse actual, int start, String expectedResultValue) {
        assertExpectedTestRows(expected, new Table(actual), start, expectedResultValue);
    }

    protected void assertExpectedTestRows(String[][] expected, Table indexable, int start, String expectedResultValue) {
        for (int i = 0; i < expected.length; i++) {
            String[] testRow = expected[i];
            for (int j = 0; j < testRow.length; j++) {
                String expectedText = testRow[j];
                String actualText = cellText(indexable, i+start, j);
                assertEqualsIgnoringTags(expectedText, actualText);
            }
            int lastCell = testRow.length;
            assertEquals(expectedResultValue,
                         cellText(indexable, i+start, lastCell));
        }
    }

    private String cellText(Table table, int row, int column) {
        return table.row(row).cell(column).text();
    }

    protected void assertEqualsIgnoringTags(String expectedText, String actualText) {
        assertTrue(expectedText.matches(TAG + actualText + TAG));
    }

    protected void assertEqual2DStringArrays(String[][] expected, String[][] actual) {
        for (int i = 0; i < actual.length; i++) {
            for (int j = 0; j < actual[i].length; j++) {
                assertEqualsIgnoringTags(actual[i][j], expected[i][j]);
            }
        }
    }

    protected void assertEqualStringArrays(String[] expected, String[] actual) {
        for (int i = 0; i < expected.length; i++) {
            assertEqualsIgnoringTags(actual[i], expected[i]);
        }
    }

    protected void assertEqualRows(String[] expected, Row row) {
        assertEquals(expected.length, row.size());
        for (int i = 0; i < expected.length; i++) {
            assertEqualsIgnoringTags(row.cell(i).text(), expected[i]);
        }
    }

    protected void assertEqualTables(String[][] expected, Table table) {
        for (int i = 0; i < expected.length; i++) {
            Row row = table.row(i);
            assertEqualRows(expected[i], row);
        }
        assertEquals(expected.length, table.size());
    }

    // silence JUnit
    public void testNothing() {}

}
